# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ObjectStorageKeyArgs', 'ObjectStorageKey']

@pulumi.input_type
class ObjectStorageKeyArgs:
    def __init__(__self__, *,
                 label: pulumi.Input[_builtins.str],
                 bucket_accesses: Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]]] = None,
                 regions: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        The set of arguments for constructing a ObjectStorageKey resource.
        :param pulumi.Input[_builtins.str] label: The label given to this key. For display purposes only.
        :param pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]] bucket_accesses: Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] regions: A set of regions where the key will grant access to create buckets.
               
               - - -
        """
        pulumi.set(__self__, "label", label)
        if bucket_accesses is not None:
            pulumi.set(__self__, "bucket_accesses", bucket_accesses)
        if regions is not None:
            pulumi.set(__self__, "regions", regions)

    @_builtins.property
    @pulumi.getter
    def label(self) -> pulumi.Input[_builtins.str]:
        """
        The label given to this key. For display purposes only.
        """
        return pulumi.get(self, "label")

    @label.setter
    def label(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "label", value)

    @_builtins.property
    @pulumi.getter(name="bucketAccesses")
    def bucket_accesses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]]]:
        """
        Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        """
        return pulumi.get(self, "bucket_accesses")

    @bucket_accesses.setter
    def bucket_accesses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]]]):
        pulumi.set(self, "bucket_accesses", value)

    @_builtins.property
    @pulumi.getter
    def regions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A set of regions where the key will grant access to create buckets.

        - - -
        """
        return pulumi.get(self, "regions")

    @regions.setter
    def regions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "regions", value)


@pulumi.input_type
class _ObjectStorageKeyState:
    def __init__(__self__, *,
                 access_key: Optional[pulumi.Input[_builtins.str]] = None,
                 bucket_accesses: Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 limited: Optional[pulumi.Input[_builtins.bool]] = None,
                 regions: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 regions_details: Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyRegionsDetailArgs']]]] = None,
                 secret_key: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering ObjectStorageKey resources.
        :param pulumi.Input[_builtins.str] access_key: This keypair's access key. This is not secret.
        :param pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]] bucket_accesses: Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        :param pulumi.Input[_builtins.str] label: The label given to this key. For display purposes only.
        :param pulumi.Input[_builtins.bool] limited: Whether or not this key is a limited access key.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] regions: A set of regions where the key will grant access to create buckets.
               
               - - -
        :param pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyRegionsDetailArgs']]] regions_details: A set of objects containing the detailed info of the regions where this key can access.
        :param pulumi.Input[_builtins.str] secret_key: This keypair's secret key.
        """
        if access_key is not None:
            pulumi.set(__self__, "access_key", access_key)
        if bucket_accesses is not None:
            pulumi.set(__self__, "bucket_accesses", bucket_accesses)
        if label is not None:
            pulumi.set(__self__, "label", label)
        if limited is not None:
            pulumi.set(__self__, "limited", limited)
        if regions is not None:
            pulumi.set(__self__, "regions", regions)
        if regions_details is not None:
            pulumi.set(__self__, "regions_details", regions_details)
        if secret_key is not None:
            pulumi.set(__self__, "secret_key", secret_key)

    @_builtins.property
    @pulumi.getter(name="accessKey")
    def access_key(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        This keypair's access key. This is not secret.
        """
        return pulumi.get(self, "access_key")

    @access_key.setter
    def access_key(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "access_key", value)

    @_builtins.property
    @pulumi.getter(name="bucketAccesses")
    def bucket_accesses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]]]:
        """
        Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        """
        return pulumi.get(self, "bucket_accesses")

    @bucket_accesses.setter
    def bucket_accesses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyBucketAccessArgs']]]]):
        pulumi.set(self, "bucket_accesses", value)

    @_builtins.property
    @pulumi.getter
    def label(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The label given to this key. For display purposes only.
        """
        return pulumi.get(self, "label")

    @label.setter
    def label(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "label", value)

    @_builtins.property
    @pulumi.getter
    def limited(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Whether or not this key is a limited access key.
        """
        return pulumi.get(self, "limited")

    @limited.setter
    def limited(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "limited", value)

    @_builtins.property
    @pulumi.getter
    def regions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A set of regions where the key will grant access to create buckets.

        - - -
        """
        return pulumi.get(self, "regions")

    @regions.setter
    def regions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "regions", value)

    @_builtins.property
    @pulumi.getter(name="regionsDetails")
    def regions_details(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyRegionsDetailArgs']]]]:
        """
        A set of objects containing the detailed info of the regions where this key can access.
        """
        return pulumi.get(self, "regions_details")

    @regions_details.setter
    def regions_details(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ObjectStorageKeyRegionsDetailArgs']]]]):
        pulumi.set(self, "regions_details", value)

    @_builtins.property
    @pulumi.getter(name="secretKey")
    def secret_key(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        This keypair's secret key.
        """
        return pulumi.get(self, "secret_key")

    @secret_key.setter
    def secret_key(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "secret_key", value)


@pulumi.type_token("linode:index/objectStorageKey:ObjectStorageKey")
class ObjectStorageKey(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 bucket_accesses: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyBucketAccessArgs', 'ObjectStorageKeyBucketAccessArgsDict']]]]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 regions: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        """
        Provides a Linode Object Storage Key resource. This can be used to create, modify, and delete Linodes Object Storage Keys.
        For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/post-object-storage-keys).

        ## Example Usage

        The following example shows how one might use this resource to create an Object Storage Key.

        ```python
        import pulumi
        import pulumi_linode as linode

        foo = linode.ObjectStorageKey("foo", label="image-access")
        ```

        The following example shows a key with limited access.

        ```python
        import pulumi
        import pulumi_linode as linode

        foobar = linode.ObjectStorageKey("foobar",
            label="my-key",
            bucket_accesses=[{
                "bucket_name": "my-bucket-name",
                "region": "us-mia",
                "permissions": "read_write",
            }])
        ```

        The following example shows how to grant a key the explicit access to multiple buckets.

        ```python
        import pulumi
        import pulumi_linode as linode

        foo = linode.ObjectStorageKey("foo",
            label="image-access",
            bucket_accesses=[
                {
                    "bucket_name": "foobar1",
                    "region": "us-mia",
                    "permissions": "read_write",
                },
                {
                    "bucket_name": "foobar2",
                    "region": "gb-lon",
                    "permissions": "read_write",
                },
            ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyBucketAccessArgs', 'ObjectStorageKeyBucketAccessArgsDict']]]] bucket_accesses: Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        :param pulumi.Input[_builtins.str] label: The label given to this key. For display purposes only.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] regions: A set of regions where the key will grant access to create buckets.
               
               - - -
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ObjectStorageKeyArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Linode Object Storage Key resource. This can be used to create, modify, and delete Linodes Object Storage Keys.
        For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/post-object-storage-keys).

        ## Example Usage

        The following example shows how one might use this resource to create an Object Storage Key.

        ```python
        import pulumi
        import pulumi_linode as linode

        foo = linode.ObjectStorageKey("foo", label="image-access")
        ```

        The following example shows a key with limited access.

        ```python
        import pulumi
        import pulumi_linode as linode

        foobar = linode.ObjectStorageKey("foobar",
            label="my-key",
            bucket_accesses=[{
                "bucket_name": "my-bucket-name",
                "region": "us-mia",
                "permissions": "read_write",
            }])
        ```

        The following example shows how to grant a key the explicit access to multiple buckets.

        ```python
        import pulumi
        import pulumi_linode as linode

        foo = linode.ObjectStorageKey("foo",
            label="image-access",
            bucket_accesses=[
                {
                    "bucket_name": "foobar1",
                    "region": "us-mia",
                    "permissions": "read_write",
                },
                {
                    "bucket_name": "foobar2",
                    "region": "gb-lon",
                    "permissions": "read_write",
                },
            ])
        ```

        :param str resource_name: The name of the resource.
        :param ObjectStorageKeyArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ObjectStorageKeyArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 bucket_accesses: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyBucketAccessArgs', 'ObjectStorageKeyBucketAccessArgsDict']]]]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 regions: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ObjectStorageKeyArgs.__new__(ObjectStorageKeyArgs)

            __props__.__dict__["bucket_accesses"] = bucket_accesses
            if label is None and not opts.urn:
                raise TypeError("Missing required property 'label'")
            __props__.__dict__["label"] = label
            __props__.__dict__["regions"] = regions
            __props__.__dict__["access_key"] = None
            __props__.__dict__["limited"] = None
            __props__.__dict__["regions_details"] = None
            __props__.__dict__["secret_key"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["secretKey"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(ObjectStorageKey, __self__).__init__(
            'linode:index/objectStorageKey:ObjectStorageKey',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            access_key: Optional[pulumi.Input[_builtins.str]] = None,
            bucket_accesses: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyBucketAccessArgs', 'ObjectStorageKeyBucketAccessArgsDict']]]]] = None,
            label: Optional[pulumi.Input[_builtins.str]] = None,
            limited: Optional[pulumi.Input[_builtins.bool]] = None,
            regions: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            regions_details: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyRegionsDetailArgs', 'ObjectStorageKeyRegionsDetailArgsDict']]]]] = None,
            secret_key: Optional[pulumi.Input[_builtins.str]] = None) -> 'ObjectStorageKey':
        """
        Get an existing ObjectStorageKey resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] access_key: This keypair's access key. This is not secret.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyBucketAccessArgs', 'ObjectStorageKeyBucketAccessArgsDict']]]] bucket_accesses: Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        :param pulumi.Input[_builtins.str] label: The label given to this key. For display purposes only.
        :param pulumi.Input[_builtins.bool] limited: Whether or not this key is a limited access key.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] regions: A set of regions where the key will grant access to create buckets.
               
               - - -
        :param pulumi.Input[Sequence[pulumi.Input[Union['ObjectStorageKeyRegionsDetailArgs', 'ObjectStorageKeyRegionsDetailArgsDict']]]] regions_details: A set of objects containing the detailed info of the regions where this key can access.
        :param pulumi.Input[_builtins.str] secret_key: This keypair's secret key.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ObjectStorageKeyState.__new__(_ObjectStorageKeyState)

        __props__.__dict__["access_key"] = access_key
        __props__.__dict__["bucket_accesses"] = bucket_accesses
        __props__.__dict__["label"] = label
        __props__.__dict__["limited"] = limited
        __props__.__dict__["regions"] = regions
        __props__.__dict__["regions_details"] = regions_details
        __props__.__dict__["secret_key"] = secret_key
        return ObjectStorageKey(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="accessKey")
    def access_key(self) -> pulumi.Output[_builtins.str]:
        """
        This keypair's access key. This is not secret.
        """
        return pulumi.get(self, "access_key")

    @_builtins.property
    @pulumi.getter(name="bucketAccesses")
    def bucket_accesses(self) -> pulumi.Output[Optional[Sequence['outputs.ObjectStorageKeyBucketAccess']]]:
        """
        Defines this key as a Limited Access Key. Limited Access Keys restrict this Object Storage key’s access to only the bucket(s) declared in this array and define their bucket-level permissions. Not providing this block will not limit this Object Storage Key.
        """
        return pulumi.get(self, "bucket_accesses")

    @_builtins.property
    @pulumi.getter
    def label(self) -> pulumi.Output[_builtins.str]:
        """
        The label given to this key. For display purposes only.
        """
        return pulumi.get(self, "label")

    @_builtins.property
    @pulumi.getter
    def limited(self) -> pulumi.Output[_builtins.bool]:
        """
        Whether or not this key is a limited access key.
        """
        return pulumi.get(self, "limited")

    @_builtins.property
    @pulumi.getter
    def regions(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        A set of regions where the key will grant access to create buckets.

        - - -
        """
        return pulumi.get(self, "regions")

    @_builtins.property
    @pulumi.getter(name="regionsDetails")
    def regions_details(self) -> pulumi.Output[Sequence['outputs.ObjectStorageKeyRegionsDetail']]:
        """
        A set of objects containing the detailed info of the regions where this key can access.
        """
        return pulumi.get(self, "regions_details")

    @_builtins.property
    @pulumi.getter(name="secretKey")
    def secret_key(self) -> pulumi.Output[_builtins.str]:
        """
        This keypair's secret key.
        """
        return pulumi.get(self, "secret_key")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetChildAccountsResult',
    'AwaitableGetChildAccountsResult',
    'get_child_accounts',
    'get_child_accounts_output',
]

@pulumi.output_type
class GetChildAccountsResult:
    """
    A collection of values returned by getChildAccounts.
    """
    def __init__(__self__, child_accounts=None, filters=None, id=None):
        if child_accounts and not isinstance(child_accounts, list):
            raise TypeError("Expected argument 'child_accounts' to be a list")
        pulumi.set(__self__, "child_accounts", child_accounts)
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)

    @_builtins.property
    @pulumi.getter(name="childAccounts")
    def child_accounts(self) -> Optional[Sequence['outputs.GetChildAccountsChildAccountResult']]:
        return pulumi.get(self, "child_accounts")

    @_builtins.property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetChildAccountsFilterResult']]:
        return pulumi.get(self, "filters")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        return pulumi.get(self, "id")


class AwaitableGetChildAccountsResult(GetChildAccountsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetChildAccountsResult(
            child_accounts=self.child_accounts,
            filters=self.filters,
            id=self.id)


def get_child_accounts(child_accounts: Optional[Sequence[Union['GetChildAccountsChildAccountArgs', 'GetChildAccountsChildAccountArgsDict']]] = None,
                       filters: Optional[Sequence[Union['GetChildAccountsFilterArgs', 'GetChildAccountsFilterArgsDict']]] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetChildAccountsResult:
    """
    Provides information about Linode Child Accounts that match a set of filters.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-child-accounts).

    **NOTE: Parent/Child related features may not be generally available.**

    ## Example Usage

    The following example shows how one might use this data source to access Child Accounts under the current Account.

    ```python
    import pulumi
    import pulumi_linode as linode

    all = linode.get_child_accounts()
    filtered = linode.get_child_accounts(filters=[
        {
            "name": "email",
            "values": ["example@linode.com"],
        },
        {
            "name": "first_name",
            "values": ["John"],
        },
        {
            "name": "last_name",
            "values": ["Smith"],
        },
    ])
    pulumi.export("allAccounts", [__item.euuid for __item in all.child_accounts])
    pulumi.export("filteredAccounts", [__item.euuid for __item in filtered.child_accounts])
    ```

    ## Filterable Fields

    * `euuid`

    * `email`

    * `first_name`

    * `last_name`

    * `company`

    * `address_1`

    * `address_2`

    * `phone`

    * `city`

    * `state`

    * `country`

    * `zip`

    * `capabilities`

    * `active_since`
    """
    __args__ = dict()
    __args__['childAccounts'] = child_accounts
    __args__['filters'] = filters
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getChildAccounts:getChildAccounts', __args__, opts=opts, typ=GetChildAccountsResult).value

    return AwaitableGetChildAccountsResult(
        child_accounts=pulumi.get(__ret__, 'child_accounts'),
        filters=pulumi.get(__ret__, 'filters'),
        id=pulumi.get(__ret__, 'id'))
def get_child_accounts_output(child_accounts: Optional[pulumi.Input[Optional[Sequence[Union['GetChildAccountsChildAccountArgs', 'GetChildAccountsChildAccountArgsDict']]]]] = None,
                              filters: Optional[pulumi.Input[Optional[Sequence[Union['GetChildAccountsFilterArgs', 'GetChildAccountsFilterArgsDict']]]]] = None,
                              opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetChildAccountsResult]:
    """
    Provides information about Linode Child Accounts that match a set of filters.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-child-accounts).

    **NOTE: Parent/Child related features may not be generally available.**

    ## Example Usage

    The following example shows how one might use this data source to access Child Accounts under the current Account.

    ```python
    import pulumi
    import pulumi_linode as linode

    all = linode.get_child_accounts()
    filtered = linode.get_child_accounts(filters=[
        {
            "name": "email",
            "values": ["example@linode.com"],
        },
        {
            "name": "first_name",
            "values": ["John"],
        },
        {
            "name": "last_name",
            "values": ["Smith"],
        },
    ])
    pulumi.export("allAccounts", [__item.euuid for __item in all.child_accounts])
    pulumi.export("filteredAccounts", [__item.euuid for __item in filtered.child_accounts])
    ```

    ## Filterable Fields

    * `euuid`

    * `email`

    * `first_name`

    * `last_name`

    * `company`

    * `address_1`

    * `address_2`

    * `phone`

    * `city`

    * `state`

    * `country`

    * `zip`

    * `capabilities`

    * `active_since`
    """
    __args__ = dict()
    __args__['childAccounts'] = child_accounts
    __args__['filters'] = filters
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getChildAccounts:getChildAccounts', __args__, opts=opts, typ=GetChildAccountsResult)
    return __ret__.apply(lambda __response__: GetChildAccountsResult(
        child_accounts=pulumi.get(__response__, 'child_accounts'),
        filters=pulumi.get(__response__, 'filters'),
        id=pulumi.get(__response__, 'id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetLkeClusterResult',
    'AwaitableGetLkeClusterResult',
    'get_lke_cluster',
    'get_lke_cluster_output',
]

@pulumi.output_type
class GetLkeClusterResult:
    """
    A collection of values returned by getLkeCluster.
    """
    def __init__(__self__, api_endpoints=None, apl_enabled=None, control_planes=None, created=None, dashboard_url=None, id=None, k8s_version=None, kubeconfig=None, label=None, pools=None, region=None, stack_type=None, status=None, subnet_id=None, tags=None, tier=None, updated=None, vpc_id=None):
        if api_endpoints and not isinstance(api_endpoints, list):
            raise TypeError("Expected argument 'api_endpoints' to be a list")
        pulumi.set(__self__, "api_endpoints", api_endpoints)
        if apl_enabled and not isinstance(apl_enabled, bool):
            raise TypeError("Expected argument 'apl_enabled' to be a bool")
        pulumi.set(__self__, "apl_enabled", apl_enabled)
        if control_planes and not isinstance(control_planes, list):
            raise TypeError("Expected argument 'control_planes' to be a list")
        pulumi.set(__self__, "control_planes", control_planes)
        if created and not isinstance(created, str):
            raise TypeError("Expected argument 'created' to be a str")
        pulumi.set(__self__, "created", created)
        if dashboard_url and not isinstance(dashboard_url, str):
            raise TypeError("Expected argument 'dashboard_url' to be a str")
        pulumi.set(__self__, "dashboard_url", dashboard_url)
        if id and not isinstance(id, int):
            raise TypeError("Expected argument 'id' to be a int")
        pulumi.set(__self__, "id", id)
        if k8s_version and not isinstance(k8s_version, str):
            raise TypeError("Expected argument 'k8s_version' to be a str")
        pulumi.set(__self__, "k8s_version", k8s_version)
        if kubeconfig and not isinstance(kubeconfig, str):
            raise TypeError("Expected argument 'kubeconfig' to be a str")
        pulumi.set(__self__, "kubeconfig", kubeconfig)
        if label and not isinstance(label, str):
            raise TypeError("Expected argument 'label' to be a str")
        pulumi.set(__self__, "label", label)
        if pools and not isinstance(pools, list):
            raise TypeError("Expected argument 'pools' to be a list")
        pulumi.set(__self__, "pools", pools)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if stack_type and not isinstance(stack_type, str):
            raise TypeError("Expected argument 'stack_type' to be a str")
        pulumi.set(__self__, "stack_type", stack_type)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)
        if subnet_id and not isinstance(subnet_id, int):
            raise TypeError("Expected argument 'subnet_id' to be a int")
        pulumi.set(__self__, "subnet_id", subnet_id)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)
        if tier and not isinstance(tier, str):
            raise TypeError("Expected argument 'tier' to be a str")
        pulumi.set(__self__, "tier", tier)
        if updated and not isinstance(updated, str):
            raise TypeError("Expected argument 'updated' to be a str")
        pulumi.set(__self__, "updated", updated)
        if vpc_id and not isinstance(vpc_id, int):
            raise TypeError("Expected argument 'vpc_id' to be a int")
        pulumi.set(__self__, "vpc_id", vpc_id)

    @_builtins.property
    @pulumi.getter(name="apiEndpoints")
    def api_endpoints(self) -> Sequence[_builtins.str]:
        """
        The endpoints for the Kubernetes API server.
        """
        return pulumi.get(self, "api_endpoints")

    @_builtins.property
    @pulumi.getter(name="aplEnabled")
    def apl_enabled(self) -> _builtins.bool:
        """
        Enables the App Platform Layer
        """
        return pulumi.get(self, "apl_enabled")

    @_builtins.property
    @pulumi.getter(name="controlPlanes")
    def control_planes(self) -> Optional[Sequence['outputs.GetLkeClusterControlPlaneResult']]:
        """
        The settings for the Kubernetes Control Plane.
        """
        return pulumi.get(self, "control_planes")

    @_builtins.property
    @pulumi.getter
    def created(self) -> _builtins.str:
        """
        When this Kubernetes cluster was created.
        """
        return pulumi.get(self, "created")

    @_builtins.property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> _builtins.str:
        """
        The Kubernetes Dashboard access URL for this cluster. LKE Enterprise does not have a dashboard URL.
        """
        return pulumi.get(self, "dashboard_url")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.int:
        """
        The ID of the node.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="k8sVersion")
    def k8s_version(self) -> _builtins.str:
        """
        The k8s version of the nodes in this Node Pool. For LKE enterprise only and may not currently available to all users even under v4beta.
        """
        return pulumi.get(self, "k8s_version")

    @_builtins.property
    @pulumi.getter
    def kubeconfig(self) -> _builtins.str:
        """
        The base64 encoded kubeconfig for the Kubernetes cluster.
        """
        return pulumi.get(self, "kubeconfig")

    @_builtins.property
    @pulumi.getter
    def label(self) -> _builtins.str:
        """
        The label of the Node Pool.
        """
        return pulumi.get(self, "label")

    @_builtins.property
    @pulumi.getter
    def pools(self) -> Optional[Sequence['outputs.GetLkeClusterPoolResult']]:
        """
        Node pools associated with this cluster.
        """
        return pulumi.get(self, "pools")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        """
        This Kubernetes cluster's location.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter(name="stackType")
    def stack_type(self) -> _builtins.str:
        """
        The networking stack type of the Kubernetes cluster.
        """
        return pulumi.get(self, "stack_type")

    @_builtins.property
    @pulumi.getter
    def status(self) -> _builtins.str:
        """
        The status of the node. (`ready`, `not_ready`)
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> _builtins.int:
        """
        The ID of the VPC subnet to use for the Kubernetes cluster. This subnet must be dual stack (IPv4 and IPv6 should both be enabled).
        """
        return pulumi.get(self, "subnet_id")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Sequence[_builtins.str]:
        """
        An array of tags applied to this object. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter
    def tier(self) -> _builtins.str:
        """
        The desired Kubernetes tier. (**Note: v4beta only and may not currently be available to all users.**)
        """
        return pulumi.get(self, "tier")

    @_builtins.property
    @pulumi.getter
    def updated(self) -> _builtins.str:
        """
        When this Kubernetes cluster was updated.
        """
        return pulumi.get(self, "updated")

    @_builtins.property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> _builtins.int:
        """
        The ID of the VPC to use for the Kubernetes cluster.
        """
        return pulumi.get(self, "vpc_id")


class AwaitableGetLkeClusterResult(GetLkeClusterResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetLkeClusterResult(
            api_endpoints=self.api_endpoints,
            apl_enabled=self.apl_enabled,
            control_planes=self.control_planes,
            created=self.created,
            dashboard_url=self.dashboard_url,
            id=self.id,
            k8s_version=self.k8s_version,
            kubeconfig=self.kubeconfig,
            label=self.label,
            pools=self.pools,
            region=self.region,
            stack_type=self.stack_type,
            status=self.status,
            subnet_id=self.subnet_id,
            tags=self.tags,
            tier=self.tier,
            updated=self.updated,
            vpc_id=self.vpc_id)


def get_lke_cluster(control_planes: Optional[Sequence[Union['GetLkeClusterControlPlaneArgs', 'GetLkeClusterControlPlaneArgsDict']]] = None,
                    id: Optional[_builtins.int] = None,
                    pools: Optional[Sequence[Union['GetLkeClusterPoolArgs', 'GetLkeClusterPoolArgsDict']]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetLkeClusterResult:
    """
    Provides details about an LKE Cluster.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-lke-cluster).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    my_cluster = linode.get_lke_cluster(id=123)
    ```


    :param Sequence[Union['GetLkeClusterControlPlaneArgs', 'GetLkeClusterControlPlaneArgsDict']] control_planes: The settings for the Kubernetes Control Plane.
    :param _builtins.int id: The LKE Cluster's ID.
    :param Sequence[Union['GetLkeClusterPoolArgs', 'GetLkeClusterPoolArgsDict']] pools: Node pools associated with this cluster.
    """
    __args__ = dict()
    __args__['controlPlanes'] = control_planes
    __args__['id'] = id
    __args__['pools'] = pools
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getLkeCluster:getLkeCluster', __args__, opts=opts, typ=GetLkeClusterResult).value

    return AwaitableGetLkeClusterResult(
        api_endpoints=pulumi.get(__ret__, 'api_endpoints'),
        apl_enabled=pulumi.get(__ret__, 'apl_enabled'),
        control_planes=pulumi.get(__ret__, 'control_planes'),
        created=pulumi.get(__ret__, 'created'),
        dashboard_url=pulumi.get(__ret__, 'dashboard_url'),
        id=pulumi.get(__ret__, 'id'),
        k8s_version=pulumi.get(__ret__, 'k8s_version'),
        kubeconfig=pulumi.get(__ret__, 'kubeconfig'),
        label=pulumi.get(__ret__, 'label'),
        pools=pulumi.get(__ret__, 'pools'),
        region=pulumi.get(__ret__, 'region'),
        stack_type=pulumi.get(__ret__, 'stack_type'),
        status=pulumi.get(__ret__, 'status'),
        subnet_id=pulumi.get(__ret__, 'subnet_id'),
        tags=pulumi.get(__ret__, 'tags'),
        tier=pulumi.get(__ret__, 'tier'),
        updated=pulumi.get(__ret__, 'updated'),
        vpc_id=pulumi.get(__ret__, 'vpc_id'))
def get_lke_cluster_output(control_planes: Optional[pulumi.Input[Optional[Sequence[Union['GetLkeClusterControlPlaneArgs', 'GetLkeClusterControlPlaneArgsDict']]]]] = None,
                           id: Optional[pulumi.Input[_builtins.int]] = None,
                           pools: Optional[pulumi.Input[Optional[Sequence[Union['GetLkeClusterPoolArgs', 'GetLkeClusterPoolArgsDict']]]]] = None,
                           opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetLkeClusterResult]:
    """
    Provides details about an LKE Cluster.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-lke-cluster).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    my_cluster = linode.get_lke_cluster(id=123)
    ```


    :param Sequence[Union['GetLkeClusterControlPlaneArgs', 'GetLkeClusterControlPlaneArgsDict']] control_planes: The settings for the Kubernetes Control Plane.
    :param _builtins.int id: The LKE Cluster's ID.
    :param Sequence[Union['GetLkeClusterPoolArgs', 'GetLkeClusterPoolArgsDict']] pools: Node pools associated with this cluster.
    """
    __args__ = dict()
    __args__['controlPlanes'] = control_planes
    __args__['id'] = id
    __args__['pools'] = pools
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getLkeCluster:getLkeCluster', __args__, opts=opts, typ=GetLkeClusterResult)
    return __ret__.apply(lambda __response__: GetLkeClusterResult(
        api_endpoints=pulumi.get(__response__, 'api_endpoints'),
        apl_enabled=pulumi.get(__response__, 'apl_enabled'),
        control_planes=pulumi.get(__response__, 'control_planes'),
        created=pulumi.get(__response__, 'created'),
        dashboard_url=pulumi.get(__response__, 'dashboard_url'),
        id=pulumi.get(__response__, 'id'),
        k8s_version=pulumi.get(__response__, 'k8s_version'),
        kubeconfig=pulumi.get(__response__, 'kubeconfig'),
        label=pulumi.get(__response__, 'label'),
        pools=pulumi.get(__response__, 'pools'),
        region=pulumi.get(__response__, 'region'),
        stack_type=pulumi.get(__response__, 'stack_type'),
        status=pulumi.get(__response__, 'status'),
        subnet_id=pulumi.get(__response__, 'subnet_id'),
        tags=pulumi.get(__response__, 'tags'),
        tier=pulumi.get(__response__, 'tier'),
        updated=pulumi.get(__response__, 'updated'),
        vpc_id=pulumi.get(__response__, 'vpc_id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['FirewallArgs', 'Firewall']

@pulumi.input_type
class FirewallArgs:
    def __init__(__self__, *,
                 inbound_policy: pulumi.Input[_builtins.str],
                 label: pulumi.Input[_builtins.str],
                 outbound_policy: pulumi.Input[_builtins.str],
                 disabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 inbounds: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]]] = None,
                 interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 linodes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 nodebalancers: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 outbounds: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None):
        """
        The set of arguments for constructing a Firewall resource.
        :param pulumi.Input[_builtins.str] inbound_policy: The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)
               
               * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[_builtins.str] label: This Firewall's unique label.
        :param pulumi.Input[_builtins.str] outbound_policy: The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        :param pulumi.Input[_builtins.bool] disabled: If `true`, the Firewall's rules are not enforced (defaults to `false`).
               
               * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]] inbounds: A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] interfaces: A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] linodes: A list of IDs of Linodes this Firewall should govern network traffic for.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] nodebalancers: A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]] outbounds: A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        pulumi.set(__self__, "inbound_policy", inbound_policy)
        pulumi.set(__self__, "label", label)
        pulumi.set(__self__, "outbound_policy", outbound_policy)
        if disabled is not None:
            pulumi.set(__self__, "disabled", disabled)
        if inbounds is not None:
            pulumi.set(__self__, "inbounds", inbounds)
        if interfaces is not None:
            pulumi.set(__self__, "interfaces", interfaces)
        if linodes is not None:
            pulumi.set(__self__, "linodes", linodes)
        if nodebalancers is not None:
            pulumi.set(__self__, "nodebalancers", nodebalancers)
        if outbounds is not None:
            pulumi.set(__self__, "outbounds", outbounds)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter(name="inboundPolicy")
    def inbound_policy(self) -> pulumi.Input[_builtins.str]:
        """
        The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)

        * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        """
        return pulumi.get(self, "inbound_policy")

    @inbound_policy.setter
    def inbound_policy(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "inbound_policy", value)

    @_builtins.property
    @pulumi.getter
    def label(self) -> pulumi.Input[_builtins.str]:
        """
        This Firewall's unique label.
        """
        return pulumi.get(self, "label")

    @label.setter
    def label(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "label", value)

    @_builtins.property
    @pulumi.getter(name="outboundPolicy")
    def outbound_policy(self) -> pulumi.Input[_builtins.str]:
        """
        The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        """
        return pulumi.get(self, "outbound_policy")

    @outbound_policy.setter
    def outbound_policy(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "outbound_policy", value)

    @_builtins.property
    @pulumi.getter
    def disabled(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If `true`, the Firewall's rules are not enforced (defaults to `false`).

        * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        """
        return pulumi.get(self, "disabled")

    @disabled.setter
    def disabled(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "disabled", value)

    @_builtins.property
    @pulumi.getter
    def inbounds(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]]]:
        """
        A firewall rule that specifies what inbound network traffic is allowed.
        """
        return pulumi.get(self, "inbounds")

    @inbounds.setter
    def inbounds(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]]]):
        pulumi.set(self, "inbounds", value)

    @_builtins.property
    @pulumi.getter
    def interfaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "interfaces")

    @interfaces.setter
    def interfaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "interfaces", value)

    @_builtins.property
    @pulumi.getter
    def linodes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of IDs of Linodes this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "linodes")

    @linodes.setter
    def linodes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "linodes", value)

    @_builtins.property
    @pulumi.getter
    def nodebalancers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "nodebalancers")

    @nodebalancers.setter
    def nodebalancers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "nodebalancers", value)

    @_builtins.property
    @pulumi.getter
    def outbounds(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]]]:
        """
        A firewall rule that specifies what outbound network traffic is allowed.
        """
        return pulumi.get(self, "outbounds")

    @outbounds.setter
    def outbounds(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]]]):
        pulumi.set(self, "outbounds", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _FirewallState:
    def __init__(__self__, *,
                 created: Optional[pulumi.Input[_builtins.str]] = None,
                 devices: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallDeviceArgs']]]] = None,
                 disabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 inbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 inbounds: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]]] = None,
                 interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 linodes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 nodebalancers: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 outbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 outbounds: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 updated: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering Firewall resources.
        :param pulumi.Input[_builtins.str] created: When this firewall was created
        :param pulumi.Input[Sequence[pulumi.Input['FirewallDeviceArgs']]] devices: The devices associated with this firewall.
        :param pulumi.Input[_builtins.bool] disabled: If `true`, the Firewall's rules are not enforced (defaults to `false`).
               
               * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[_builtins.str] inbound_policy: The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)
               
               * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]] inbounds: A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] interfaces: A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        :param pulumi.Input[_builtins.str] label: This Firewall's unique label.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] linodes: A list of IDs of Linodes this Firewall should govern network traffic for.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] nodebalancers: A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        :param pulumi.Input[_builtins.str] outbound_policy: The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        :param pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]] outbounds: A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[_builtins.str] status: The status of the Firewall.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        :param pulumi.Input[_builtins.str] updated: When this firewall was last updated
        """
        if created is not None:
            pulumi.set(__self__, "created", created)
        if devices is not None:
            pulumi.set(__self__, "devices", devices)
        if disabled is not None:
            pulumi.set(__self__, "disabled", disabled)
        if inbound_policy is not None:
            pulumi.set(__self__, "inbound_policy", inbound_policy)
        if inbounds is not None:
            pulumi.set(__self__, "inbounds", inbounds)
        if interfaces is not None:
            pulumi.set(__self__, "interfaces", interfaces)
        if label is not None:
            pulumi.set(__self__, "label", label)
        if linodes is not None:
            pulumi.set(__self__, "linodes", linodes)
        if nodebalancers is not None:
            pulumi.set(__self__, "nodebalancers", nodebalancers)
        if outbound_policy is not None:
            pulumi.set(__self__, "outbound_policy", outbound_policy)
        if outbounds is not None:
            pulumi.set(__self__, "outbounds", outbounds)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if updated is not None:
            pulumi.set(__self__, "updated", updated)

    @_builtins.property
    @pulumi.getter
    def created(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        When this firewall was created
        """
        return pulumi.get(self, "created")

    @created.setter
    def created(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "created", value)

    @_builtins.property
    @pulumi.getter
    def devices(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallDeviceArgs']]]]:
        """
        The devices associated with this firewall.
        """
        return pulumi.get(self, "devices")

    @devices.setter
    def devices(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallDeviceArgs']]]]):
        pulumi.set(self, "devices", value)

    @_builtins.property
    @pulumi.getter
    def disabled(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If `true`, the Firewall's rules are not enforced (defaults to `false`).

        * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        """
        return pulumi.get(self, "disabled")

    @disabled.setter
    def disabled(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "disabled", value)

    @_builtins.property
    @pulumi.getter(name="inboundPolicy")
    def inbound_policy(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)

        * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        """
        return pulumi.get(self, "inbound_policy")

    @inbound_policy.setter
    def inbound_policy(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "inbound_policy", value)

    @_builtins.property
    @pulumi.getter
    def inbounds(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]]]:
        """
        A firewall rule that specifies what inbound network traffic is allowed.
        """
        return pulumi.get(self, "inbounds")

    @inbounds.setter
    def inbounds(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallInboundArgs']]]]):
        pulumi.set(self, "inbounds", value)

    @_builtins.property
    @pulumi.getter
    def interfaces(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "interfaces")

    @interfaces.setter
    def interfaces(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "interfaces", value)

    @_builtins.property
    @pulumi.getter
    def label(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        This Firewall's unique label.
        """
        return pulumi.get(self, "label")

    @label.setter
    def label(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "label", value)

    @_builtins.property
    @pulumi.getter
    def linodes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of IDs of Linodes this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "linodes")

    @linodes.setter
    def linodes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "linodes", value)

    @_builtins.property
    @pulumi.getter
    def nodebalancers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]:
        """
        A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "nodebalancers")

    @nodebalancers.setter
    def nodebalancers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]]):
        pulumi.set(self, "nodebalancers", value)

    @_builtins.property
    @pulumi.getter(name="outboundPolicy")
    def outbound_policy(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        """
        return pulumi.get(self, "outbound_policy")

    @outbound_policy.setter
    def outbound_policy(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "outbound_policy", value)

    @_builtins.property
    @pulumi.getter
    def outbounds(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]]]:
        """
        A firewall rule that specifies what outbound network traffic is allowed.
        """
        return pulumi.get(self, "outbounds")

    @outbounds.setter
    def outbounds(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallOutboundArgs']]]]):
        pulumi.set(self, "outbounds", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The status of the Firewall.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def updated(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        When this firewall was last updated
        """
        return pulumi.get(self, "updated")

    @updated.setter
    def updated(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "updated", value)


@pulumi.type_token("linode:index/firewall:Firewall")
class Firewall(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 disabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 inbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 inbounds: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallInboundArgs', 'FirewallInboundArgsDict']]]]] = None,
                 interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 linodes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 nodebalancers: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 outbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 outbounds: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallOutboundArgs', 'FirewallOutboundArgsDict']]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        """
        Manages a Linode Firewall.
        For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/post-firewalls).

        ## Example Usage

        Accept only inbound HTTP(s) requests and drop outbound HTTP(s) requests:

        ```python
        import pulumi
        import pulumi_linode as linode

        my_instance = linode.Instance("my_instance",
            label="my_instance",
            image="linode/ubuntu22.04",
            region="us-southeast",
            type="g6-standard-1",
            root_pass="bogusPassword$",
            swap_size=256)
        my_firewall = linode.Firewall("my_firewall",
            label="my_firewall",
            inbounds=[
                {
                    "label": "allow-http",
                    "action": "ACCEPT",
                    "protocol": "TCP",
                    "ports": "80",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
                {
                    "label": "allow-https",
                    "action": "ACCEPT",
                    "protocol": "TCP",
                    "ports": "443",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
            ],
            inbound_policy="DROP",
            outbounds=[
                {
                    "label": "reject-http",
                    "action": "DROP",
                    "protocol": "TCP",
                    "ports": "80",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
                {
                    "label": "reject-https",
                    "action": "DROP",
                    "protocol": "TCP",
                    "ports": "443",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
            ],
            outbound_policy="ACCEPT",
            linodes=[my_instance.id])
        ```

        ## Import

        Firewalls can be imported using the `id`, e.g.

        ```sh
        $ pulumi import linode:index/firewall:Firewall my_firewall 12345
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.bool] disabled: If `true`, the Firewall's rules are not enforced (defaults to `false`).
               
               * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[_builtins.str] inbound_policy: The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)
               
               * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[Union['FirewallInboundArgs', 'FirewallInboundArgsDict']]]] inbounds: A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] interfaces: A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        :param pulumi.Input[_builtins.str] label: This Firewall's unique label.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] linodes: A list of IDs of Linodes this Firewall should govern network traffic for.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] nodebalancers: A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        :param pulumi.Input[_builtins.str] outbound_policy: The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        :param pulumi.Input[Sequence[pulumi.Input[Union['FirewallOutboundArgs', 'FirewallOutboundArgsDict']]]] outbounds: A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FirewallArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a Linode Firewall.
        For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/post-firewalls).

        ## Example Usage

        Accept only inbound HTTP(s) requests and drop outbound HTTP(s) requests:

        ```python
        import pulumi
        import pulumi_linode as linode

        my_instance = linode.Instance("my_instance",
            label="my_instance",
            image="linode/ubuntu22.04",
            region="us-southeast",
            type="g6-standard-1",
            root_pass="bogusPassword$",
            swap_size=256)
        my_firewall = linode.Firewall("my_firewall",
            label="my_firewall",
            inbounds=[
                {
                    "label": "allow-http",
                    "action": "ACCEPT",
                    "protocol": "TCP",
                    "ports": "80",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
                {
                    "label": "allow-https",
                    "action": "ACCEPT",
                    "protocol": "TCP",
                    "ports": "443",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
            ],
            inbound_policy="DROP",
            outbounds=[
                {
                    "label": "reject-http",
                    "action": "DROP",
                    "protocol": "TCP",
                    "ports": "80",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
                {
                    "label": "reject-https",
                    "action": "DROP",
                    "protocol": "TCP",
                    "ports": "443",
                    "ipv4s": ["0.0.0.0/0"],
                    "ipv6s": ["::/0"],
                },
            ],
            outbound_policy="ACCEPT",
            linodes=[my_instance.id])
        ```

        ## Import

        Firewalls can be imported using the `id`, e.g.

        ```sh
        $ pulumi import linode:index/firewall:Firewall my_firewall 12345
        ```

        :param str resource_name: The name of the resource.
        :param FirewallArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FirewallArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 disabled: Optional[pulumi.Input[_builtins.bool]] = None,
                 inbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 inbounds: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallInboundArgs', 'FirewallInboundArgsDict']]]]] = None,
                 interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 label: Optional[pulumi.Input[_builtins.str]] = None,
                 linodes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 nodebalancers: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
                 outbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 outbounds: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallOutboundArgs', 'FirewallOutboundArgsDict']]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FirewallArgs.__new__(FirewallArgs)

            __props__.__dict__["disabled"] = disabled
            if inbound_policy is None and not opts.urn:
                raise TypeError("Missing required property 'inbound_policy'")
            __props__.__dict__["inbound_policy"] = inbound_policy
            __props__.__dict__["inbounds"] = inbounds
            __props__.__dict__["interfaces"] = interfaces
            if label is None and not opts.urn:
                raise TypeError("Missing required property 'label'")
            __props__.__dict__["label"] = label
            __props__.__dict__["linodes"] = linodes
            __props__.__dict__["nodebalancers"] = nodebalancers
            if outbound_policy is None and not opts.urn:
                raise TypeError("Missing required property 'outbound_policy'")
            __props__.__dict__["outbound_policy"] = outbound_policy
            __props__.__dict__["outbounds"] = outbounds
            __props__.__dict__["tags"] = tags
            __props__.__dict__["created"] = None
            __props__.__dict__["devices"] = None
            __props__.__dict__["status"] = None
            __props__.__dict__["updated"] = None
        super(Firewall, __self__).__init__(
            'linode:index/firewall:Firewall',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            created: Optional[pulumi.Input[_builtins.str]] = None,
            devices: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallDeviceArgs', 'FirewallDeviceArgsDict']]]]] = None,
            disabled: Optional[pulumi.Input[_builtins.bool]] = None,
            inbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
            inbounds: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallInboundArgs', 'FirewallInboundArgsDict']]]]] = None,
            interfaces: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
            label: Optional[pulumi.Input[_builtins.str]] = None,
            linodes: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
            nodebalancers: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.int]]]] = None,
            outbound_policy: Optional[pulumi.Input[_builtins.str]] = None,
            outbounds: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FirewallOutboundArgs', 'FirewallOutboundArgsDict']]]]] = None,
            status: Optional[pulumi.Input[_builtins.str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            updated: Optional[pulumi.Input[_builtins.str]] = None) -> 'Firewall':
        """
        Get an existing Firewall resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] created: When this firewall was created
        :param pulumi.Input[Sequence[pulumi.Input[Union['FirewallDeviceArgs', 'FirewallDeviceArgsDict']]]] devices: The devices associated with this firewall.
        :param pulumi.Input[_builtins.bool] disabled: If `true`, the Firewall's rules are not enforced (defaults to `false`).
               
               * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[_builtins.str] inbound_policy: The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)
               
               * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[Union['FirewallInboundArgs', 'FirewallInboundArgsDict']]]] inbounds: A firewall rule that specifies what inbound network traffic is allowed.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] interfaces: A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        :param pulumi.Input[_builtins.str] label: This Firewall's unique label.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] linodes: A list of IDs of Linodes this Firewall should govern network traffic for.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.int]]] nodebalancers: A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        :param pulumi.Input[_builtins.str] outbound_policy: The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        :param pulumi.Input[Sequence[pulumi.Input[Union['FirewallOutboundArgs', 'FirewallOutboundArgsDict']]]] outbounds: A firewall rule that specifies what outbound network traffic is allowed.
        :param pulumi.Input[_builtins.str] status: The status of the Firewall.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        :param pulumi.Input[_builtins.str] updated: When this firewall was last updated
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FirewallState.__new__(_FirewallState)

        __props__.__dict__["created"] = created
        __props__.__dict__["devices"] = devices
        __props__.__dict__["disabled"] = disabled
        __props__.__dict__["inbound_policy"] = inbound_policy
        __props__.__dict__["inbounds"] = inbounds
        __props__.__dict__["interfaces"] = interfaces
        __props__.__dict__["label"] = label
        __props__.__dict__["linodes"] = linodes
        __props__.__dict__["nodebalancers"] = nodebalancers
        __props__.__dict__["outbound_policy"] = outbound_policy
        __props__.__dict__["outbounds"] = outbounds
        __props__.__dict__["status"] = status
        __props__.__dict__["tags"] = tags
        __props__.__dict__["updated"] = updated
        return Firewall(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def created(self) -> pulumi.Output[_builtins.str]:
        """
        When this firewall was created
        """
        return pulumi.get(self, "created")

    @_builtins.property
    @pulumi.getter
    def devices(self) -> pulumi.Output[Sequence['outputs.FirewallDevice']]:
        """
        The devices associated with this firewall.
        """
        return pulumi.get(self, "devices")

    @_builtins.property
    @pulumi.getter
    def disabled(self) -> pulumi.Output[_builtins.bool]:
        """
        If `true`, the Firewall's rules are not enforced (defaults to `false`).

        * `inbound` - (Optional) A firewall rule that specifies what inbound network traffic is allowed.
        """
        return pulumi.get(self, "disabled")

    @_builtins.property
    @pulumi.getter(name="inboundPolicy")
    def inbound_policy(self) -> pulumi.Output[_builtins.str]:
        """
        The default behavior for inbound traffic. This setting can be overridden by updating the inbound.action property of the Firewall Rule. (`ACCEPT`, `DROP`)

        * `outbound` - (Optional) A firewall rule that specifies what outbound network traffic is allowed.
        """
        return pulumi.get(self, "inbound_policy")

    @_builtins.property
    @pulumi.getter
    def inbounds(self) -> pulumi.Output[Optional[Sequence['outputs.FirewallInbound']]]:
        """
        A firewall rule that specifies what inbound network traffic is allowed.
        """
        return pulumi.get(self, "inbounds")

    @_builtins.property
    @pulumi.getter
    def interfaces(self) -> pulumi.Output[Sequence[_builtins.int]]:
        """
        A list of IDs of Linode Interfaces this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "interfaces")

    @_builtins.property
    @pulumi.getter
    def label(self) -> pulumi.Output[_builtins.str]:
        """
        This Firewall's unique label.
        """
        return pulumi.get(self, "label")

    @_builtins.property
    @pulumi.getter
    def linodes(self) -> pulumi.Output[Sequence[_builtins.int]]:
        """
        A list of IDs of Linodes this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "linodes")

    @_builtins.property
    @pulumi.getter
    def nodebalancers(self) -> pulumi.Output[Sequence[_builtins.int]]:
        """
        A list of IDs of NodeBalancers this Firewall should govern network traffic for.
        """
        return pulumi.get(self, "nodebalancers")

    @_builtins.property
    @pulumi.getter(name="outboundPolicy")
    def outbound_policy(self) -> pulumi.Output[_builtins.str]:
        """
        The default behavior for outbound traffic. This setting can be overridden by updating the outbound.action property for an individual Firewall Rule. (`ACCEPT`, `DROP`)
        """
        return pulumi.get(self, "outbound_policy")

    @_builtins.property
    @pulumi.getter
    def outbounds(self) -> pulumi.Output[Optional[Sequence['outputs.FirewallOutbound']]]:
        """
        A firewall rule that specifies what outbound network traffic is allowed.
        """
        return pulumi.get(self, "outbounds")

    @_builtins.property
    @pulumi.getter
    def status(self) -> pulumi.Output[_builtins.str]:
        """
        The status of the Firewall.
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        A list of tags applied to the Kubernetes cluster. Tags are case-insensitive and are for organizational purposes only.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter
    def updated(self) -> pulumi.Output[_builtins.str]:
        """
        When this firewall was last updated
        """
        return pulumi.get(self, "updated")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetInstanceNetworkingResult',
    'AwaitableGetInstanceNetworkingResult',
    'get_instance_networking',
    'get_instance_networking_output',
]

@pulumi.output_type
class GetInstanceNetworkingResult:
    """
    A collection of values returned by getInstanceNetworking.
    """
    def __init__(__self__, id=None, ipv4s=None, ipv6s=None, linode_id=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ipv4s and not isinstance(ipv4s, list):
            raise TypeError("Expected argument 'ipv4s' to be a list")
        pulumi.set(__self__, "ipv4s", ipv4s)
        if ipv6s and not isinstance(ipv6s, list):
            raise TypeError("Expected argument 'ipv6s' to be a list")
        pulumi.set(__self__, "ipv6s", ipv6s)
        if linode_id and not isinstance(linode_id, int):
            raise TypeError("Expected argument 'linode_id' to be a int")
        pulumi.set(__self__, "linode_id", linode_id)

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def ipv4s(self) -> Sequence['outputs.GetInstanceNetworkingIpv4Result']:
        return pulumi.get(self, "ipv4s")

    @_builtins.property
    @pulumi.getter
    def ipv6s(self) -> Sequence['outputs.GetInstanceNetworkingIpv6Result']:
        return pulumi.get(self, "ipv6s")

    @_builtins.property
    @pulumi.getter(name="linodeId")
    def linode_id(self) -> _builtins.int:
        """
        The ID of the Linode this address currently belongs to.
        """
        return pulumi.get(self, "linode_id")


class AwaitableGetInstanceNetworkingResult(GetInstanceNetworkingResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetInstanceNetworkingResult(
            id=self.id,
            ipv4s=self.ipv4s,
            ipv6s=self.ipv6s,
            linode_id=self.linode_id)


def get_instance_networking(linode_id: Optional[_builtins.int] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetInstanceNetworkingResult:
    """
    Provides details about the networking configuration of an Instance.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-linode-config-interfaces).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    example = linode.get_instance_networking(linode_id=123)
    ```


    :param _builtins.int linode_id: The Linode instance's ID.
    """
    __args__ = dict()
    __args__['linodeId'] = linode_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getInstanceNetworking:getInstanceNetworking', __args__, opts=opts, typ=GetInstanceNetworkingResult).value

    return AwaitableGetInstanceNetworkingResult(
        id=pulumi.get(__ret__, 'id'),
        ipv4s=pulumi.get(__ret__, 'ipv4s'),
        ipv6s=pulumi.get(__ret__, 'ipv6s'),
        linode_id=pulumi.get(__ret__, 'linode_id'))
def get_instance_networking_output(linode_id: Optional[pulumi.Input[_builtins.int]] = None,
                                   opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetInstanceNetworkingResult]:
    """
    Provides details about the networking configuration of an Instance.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-linode-config-interfaces).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_linode as linode

    example = linode.get_instance_networking(linode_id=123)
    ```


    :param _builtins.int linode_id: The Linode instance's ID.
    """
    __args__ = dict()
    __args__['linodeId'] = linode_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getInstanceNetworking:getInstanceNetworking', __args__, opts=opts, typ=GetInstanceNetworkingResult)
    return __ret__.apply(lambda __response__: GetInstanceNetworkingResult(
        id=pulumi.get(__response__, 'id'),
        ipv4s=pulumi.get(__response__, 'ipv4s'),
        ipv6s=pulumi.get(__response__, 'ipv6s'),
        linode_id=pulumi.get(__response__, 'linode_id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetLkeVersionsResult',
    'AwaitableGetLkeVersionsResult',
    'get_lke_versions',
    'get_lke_versions_output',
]

@pulumi.output_type
class GetLkeVersionsResult:
    """
    A collection of values returned by getLkeVersions.
    """
    def __init__(__self__, id=None, tier=None, versions=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if tier and not isinstance(tier, str):
            raise TypeError("Expected argument 'tier' to be a str")
        pulumi.set(__self__, "tier", tier)
        if versions and not isinstance(versions, list):
            raise TypeError("Expected argument 'versions' to be a list")
        pulumi.set(__self__, "versions", versions)

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The Kubernetes version numbers available for deployment to a Kubernetes cluster in the format of [major].[minor], and the latest supported patch version.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def tier(self) -> Optional[_builtins.str]:
        """
        The Kubernetes version tier. Only exported if `tier` was provided when using the datasource.
        """
        return pulumi.get(self, "tier")

    @_builtins.property
    @pulumi.getter
    def versions(self) -> Optional[Sequence['outputs.GetLkeVersionsVersionResult']]:
        return pulumi.get(self, "versions")


class AwaitableGetLkeVersionsResult(GetLkeVersionsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetLkeVersionsResult(
            id=self.id,
            tier=self.tier,
            versions=self.versions)


def get_lke_versions(tier: Optional[_builtins.str] = None,
                     versions: Optional[Sequence[Union['GetLkeVersionsVersionArgs', 'GetLkeVersionsVersionArgsDict']]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetLkeVersionsResult:
    """
    Provides details about the Kubernetes versions available for deployment to a Kubernetes cluster.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-lke-versions).

    ## Example Usage

    The following example shows how one might use this data source to access information about a Linode LKE Version.

    ```python
    import pulumi
    import pulumi_linode as linode

    example = linode.get_lke_versions()
    pulumi.export("exampleOutput", example)
    pulumi.export("exampleOutputFirstVersion", example.versions[0])
    ```

    The following example shows how one might use this data source to access information about a Linode LKE Version
    with additional information about the Linode LKE Version's tier (`enterprise` or `standard`).

    > **_NOTE:_**  This functionality may not be currently available to all users and can only be used with v4beta.

    ```python
    import pulumi
    import pulumi_linode as linode

    example_enterprise = linode.get_lke_versions(tier="enterprise")
    pulumi.export("exampleEnterpriseOutput", example_enterprise)
    pulumi.export("exampleEnterpriseOutputFirstVersion", example_enterprise.versions[0])
    ```


    :param _builtins.str tier: The tier (`standard` or `enterprise`) of Linode LKE Versions to fetch.
    """
    __args__ = dict()
    __args__['tier'] = tier
    __args__['versions'] = versions
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('linode:index/getLkeVersions:getLkeVersions', __args__, opts=opts, typ=GetLkeVersionsResult).value

    return AwaitableGetLkeVersionsResult(
        id=pulumi.get(__ret__, 'id'),
        tier=pulumi.get(__ret__, 'tier'),
        versions=pulumi.get(__ret__, 'versions'))
def get_lke_versions_output(tier: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                            versions: Optional[pulumi.Input[Optional[Sequence[Union['GetLkeVersionsVersionArgs', 'GetLkeVersionsVersionArgsDict']]]]] = None,
                            opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetLkeVersionsResult]:
    """
    Provides details about the Kubernetes versions available for deployment to a Kubernetes cluster.
    For more information, see the [Linode APIv4 docs](https://techdocs.akamai.com/linode-api/reference/get-lke-versions).

    ## Example Usage

    The following example shows how one might use this data source to access information about a Linode LKE Version.

    ```python
    import pulumi
    import pulumi_linode as linode

    example = linode.get_lke_versions()
    pulumi.export("exampleOutput", example)
    pulumi.export("exampleOutputFirstVersion", example.versions[0])
    ```

    The following example shows how one might use this data source to access information about a Linode LKE Version
    with additional information about the Linode LKE Version's tier (`enterprise` or `standard`).

    > **_NOTE:_**  This functionality may not be currently available to all users and can only be used with v4beta.

    ```python
    import pulumi
    import pulumi_linode as linode

    example_enterprise = linode.get_lke_versions(tier="enterprise")
    pulumi.export("exampleEnterpriseOutput", example_enterprise)
    pulumi.export("exampleEnterpriseOutputFirstVersion", example_enterprise.versions[0])
    ```


    :param _builtins.str tier: The tier (`standard` or `enterprise`) of Linode LKE Versions to fetch.
    """
    __args__ = dict()
    __args__['tier'] = tier
    __args__['versions'] = versions
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('linode:index/getLkeVersions:getLkeVersions', __args__, opts=opts, typ=GetLkeVersionsResult)
    return __ret__.apply(lambda __response__: GetLkeVersionsResult(
        id=pulumi.get(__response__, 'id'),
        tier=pulumi.get(__response__, 'tier'),
        versions=pulumi.get(__response__, 'versions')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

import types

__config__ = pulumi.Config('linode')


class _ExportableConfig(types.ModuleType):
    @_builtins.property
    def api_ca_path(self) -> Optional[str]:
        """
        The path to a Linode API CA file to trust.
        """
        return __config__.get('apiCaPath')

    @_builtins.property
    def api_version(self) -> Optional[str]:
        """
        The version of Linode API.
        """
        return __config__.get('apiVersion') or _utilities.get_env('LINODE_API_VERSION')

    @_builtins.property
    def config_path(self) -> Optional[str]:
        """
        The path to the Linode config file to use. (default `~/.config/linode`)
        """
        return __config__.get('configPath')

    @_builtins.property
    def config_profile(self) -> Optional[str]:
        """
        The Linode config profile to use. (default `default`)
        """
        return __config__.get('configProfile')

    @_builtins.property
    def disable_internal_cache(self) -> Optional[bool]:
        """
        Disable the internal caching system that backs certain Linode API requests.
        """
        return __config__.get_bool('disableInternalCache')

    @_builtins.property
    def event_poll_ms(self) -> Optional[int]:
        """
        The rate in milliseconds to poll for events.
        """
        return __config__.get_int('eventPollMs')

    @_builtins.property
    def lke_event_poll_ms(self) -> Optional[int]:
        """
        The rate in milliseconds to poll for LKE events.
        """
        return __config__.get_int('lkeEventPollMs')

    @_builtins.property
    def lke_node_ready_poll_ms(self) -> Optional[int]:
        """
        The rate in milliseconds to poll for an LKE node to be ready.
        """
        return __config__.get_int('lkeNodeReadyPollMs')

    @_builtins.property
    def max_retry_delay_ms(self) -> Optional[int]:
        """
        Maximum delay in milliseconds before retrying a request.
        """
        return __config__.get_int('maxRetryDelayMs')

    @_builtins.property
    def min_retry_delay_ms(self) -> Optional[int]:
        """
        Minimum delay in milliseconds before retrying a request.
        """
        return __config__.get_int('minRetryDelayMs')

    @_builtins.property
    def obj_access_key(self) -> Optional[str]:
        """
        The access key to be used in ObjectStorageBucket and linode_object_storage_object.
        """
        return __config__.get('objAccessKey')

    @_builtins.property
    def obj_bucket_force_delete(self) -> Optional[bool]:
        """
        If true, when deleting a ObjectStorageBucket any objects and versions will be force deleted.
        """
        return __config__.get_bool('objBucketForceDelete')

    @_builtins.property
    def obj_secret_key(self) -> Optional[str]:
        """
        The secret key to be used in ObjectStorageBucket and linode_object_storage_object.
        """
        return __config__.get('objSecretKey')

    @_builtins.property
    def obj_use_temp_keys(self) -> Optional[bool]:
        """
        If true, temporary object keys will be created implicitly at apply-time for the ObjectStorageObject and linode_object_sorage_bucket resource.
        """
        return __config__.get_bool('objUseTempKeys')

    @_builtins.property
    def skip_implicit_reboots(self) -> Optional[bool]:
        """
        If true, Linode Instances will not be rebooted on config and interface changes.
        """
        return __config__.get_bool('skipImplicitReboots')

    @_builtins.property
    def skip_instance_delete_poll(self) -> Optional[bool]:
        """
        Skip waiting for a Instance resource to finish deleting.
        """
        return __config__.get_bool('skipInstanceDeletePoll')

    @_builtins.property
    def skip_instance_ready_poll(self) -> Optional[bool]:
        """
        Skip waiting for a Instance resource to be running.
        """
        return __config__.get_bool('skipInstanceReadyPoll')

    @_builtins.property
    def token(self) -> Optional[str]:
        """
        The token that allows you access to your Linode account
        """
        return __config__.get('token')

    @_builtins.property
    def ua_prefix(self) -> Optional[str]:
        """
        An HTTP User-Agent Prefix to prepend in API requests.
        """
        return __config__.get('uaPrefix') or _utilities.get_env('LINODE_UA_PREFIX')

    @_builtins.property
    def url(self) -> Optional[str]:
        """
        The HTTP(S) API address of the Linode API to use.
        """
        return __config__.get('url') or _utilities.get_env('LINODE_URL')

